/* Copyright (C) 2012 mbed.org, MIT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#include "Socket/Socket.h"
#include <cstring>

using std::memset;

Socket::Socket() : _sock_fd(-1) {
    
}

int Socket::init(int type) {
    if (_sock_fd != -1)
        return -1;
    
    int fd = lwip_socket(AF_INET, type, 0);
    if (fd < 0)
        return -1;
    
    _sock_fd = fd;
    return 0;
}


void Socket::set_timeout(int timeout) {
    _timeout.tv_sec = timeout / 1000;
    _timeout.tv_usec = (timeout - (_timeout.tv_sec * 1000)) * 1000;
}

int Socket::select(fd_set* readset, fd_set* writeset) {
    if ((_timeout.tv_sec == 0) && (_timeout.tv_usec == 0))
        return 0;
    
    FD_ZERO(&_fdSet);
    FD_SET(_sock_fd, &_fdSet);
    
    int ret = lwip_select(FD_SETSIZE, readset, writeset, NULL, &_timeout);
    return (ret <= 0 || !FD_ISSET(_sock_fd, &_fdSet)) ? (-1) : (0);
}

int Socket::wait_readable(void) {
    return select(&_fdSet, NULL);
}

int Socket::wait_writable(void) {
    return select(NULL, &_fdSet);
}

int Socket::close() {
    if (_sock_fd < 0)
        return -1;
    
    lwip_close(_sock_fd);
    _sock_fd = -1;
    
    return 0;
}
